#include "../C-Engine/CEngine/Object.hpp"
#include "../C-Engine/CEngine/Engine.hpp"
#include "../C-Engine/CEngine/Script.hpp"

#include "../C-Engine/CPhysics/Transforms.hpp"
#include "../C-Engine/CPhysics/Rigibodys.hpp"

#include "../C-Engine/CRendering/Render.hpp"
#include "../C-Engine/CRendering/Animation.hpp"
#include "../C-Engine/CRendering/Sprite.hpp"
#include "../C-Engine/CRendering/TilesMapping.hpp"

extern "C"
{
#include <fxlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../C-Engine/CExtraLib/input.h"
#include "../C-Engine/CExtraLib/MonochromeLib.h"
#include "../C-Engine/CExtraLib/FonctionC.h"
}

Engine::Engine()
{
    this->execute = true;
    this->run = true;
    this->fps = false;
    this->fpswish = 2;
    this->ListeObject = NULL;
    this->EScript = NULL;
    this->restart = false;
    this->time = 0;
    this->nbobject = 0;
    this->gamemap = NULL;
    this->type = 0;

    screen.min.x = 0;
    screen.max.x = 127;
    screen.min.y = 0;
    screen.max.y = 63;

}

void Engine::SetType(int v)
{
    this->type = v;
}

int Engine::GetType()
{
    return this->type;
}
    void Engine::PlayGame()
    {
        this->run = true;
    }
    void Engine::PauseGame(){
        this->run = false;
    }
    void Engine::StartGame(){
        this->execute = true;
    }
    void Engine::StopGame(){
        this->execute = false;
    }
    void Engine::SetFpsWish(int v){
        this->fpswish = v;
    }
    void Engine::ShowFps(){
        this->fps = true;
    }
    void Engine::HideFps(){
        this->fps = false;
    }
    void Engine::RestartGame(){
        this->restart = true;
    }

void Engine::AddObject( Object * v)
{
    Object ** Tab = ListeObject;

    ListeObject = (Object**)malloc(sizeof(Object *) * (nbobject+1));

    for(int i = 0 ; i < nbobject  ; i++)if(Tab[i] != NULL)ListeObject[i] = Tab[i];

    this->ListeObject[ nbobject ] = v;

    nbobject ++;

    delete Tab;

    v->AffectEngine(this);
}

void Engine::DelObject( Object * v)
{
    Object ** Tab;
    int index = -1;

    Tab = this->ListeObject;

    if(Tab == NULL)
        return ;

    for(int i = 0 ; i < nbobject  ; i++)
    {
        if(Tab[i] == v ) index = i ;
    }

    if(index == -1)return;

    this->ListeObject = (Object**)malloc(sizeof(Object *) * (nbobject-1));

    if( index > 0)
         {
            for(int i = 0 ; i < index  ; i++)
            {
                this->ListeObject[ i ] = Tab[ i ];
            }
         }

     for(int i = index + 1 ; i < nbobject  ; i++)
        {
            this->ListeObject[ i - 1] = Tab[ i ];
        }

    nbobject --;

    delete v;

    delete Tab;
}

void Engine::CreateTab( int v)
{
    this->ListeObject = new Object * [v];
    this->nbobject = v;
}

void Engine::AffectScript(Script & VScript)
{
    this->EScript = &VScript;
    EScript->AffectEngine(this);
}

void Engine::AffectMap(Map * VMap)
{
    this->gamemap = VMap;
}

bool Engine::WaitOneSecond()
{
    if(RTC_getTicks() - time > 128) // si il s'est coul une seconde complte
    {
        time = RTC_getTicks(); // et on se rappelle du nombre de ticks de la dernire seconde coule
        return true;
    }

return false;

}

void Engine::UpdateRelativePosition()
{
     for(int i = 0; i< nbobject; i++)
        {
        if(ListeObject[i]->GetEnable() == true) ListeObject[i]->OTransforms->SetRelativeXY( ListeObject[i]->OTransforms->GetX() - this->screen.min.x , ListeObject[i]->OTransforms->GetY() - this->screen.min.y );
        }
}

void Engine::MoveScreen(int x , int y)
{
    int w = screen.max.x - screen.min.x;
    int h = screen.max.y - screen.min.y;

    screen.min.x = x;
    screen.max.x = x + w;
    screen.min.y = y;
    screen.max.y = y + h;
}

void Engine::MiddleScreen(int x , int y , bool fixe)
{
    int w = screen.max.x - screen.min.x;
    int h = screen.max.y - screen.min.y;

    screen.min.x = x - w/2;
    screen.max.x = x + w/2;
    screen.min.y = y - h/2;
    screen.max.y = y + h/2;

    if(screen.min.x < 0 && fixe)
    {
        screen.min.x = 0;
        screen.max.x = w;
    }

    if(gamemap != NULL && fixe)
    {
        if(screen.max.x > gamemap->nbtiles_largeur_monde * gamemap->LARGEUR_TILE)
        {
            screen.max.x = gamemap->nbtiles_largeur_monde * gamemap->LARGEUR_TILE;
            screen.min.x = screen.max.x - w;
        }

        if(screen.min.y < 0)
        {
            screen.min.y = 0;
            screen.max.y = h;
        }

        if(screen.max.y > gamemap->nbtiles_hauteur_monde * gamemap->HAUTEUR_TILE)
        {
            screen.max.y = gamemap->nbtiles_hauteur_monde * gamemap->HAUTEUR_TILE;
            screen.min.y = screen.max.y - h;
        }
    }
}

int Engine::Game()
{

srand(RTC_getTicks());

if(EScript != NULL)EScript->Start();

    do
    {
    this->restart = false;

    for(int i = 0; i< nbobject; i++)
        {
            if(ListeObject[i]->OScript != NULL && ListeObject[i]->GetEnable() == true)ListeObject[i]->OScript->Start();
        }

    while(this->execute == true && this->restart == false)
        {
        while( this->run == true && this->execute == true && this->restart == false)
            {
            ML_clear_vram();
            input_update();

            if(EScript != NULL)EScript->Update();

            if(WaitOneSecond())
            {
                if(EScript != NULL)EScript->UpdateEverySecond();

                for(int i = 0; i < nbobject; i++)
                {
                    if(ListeObject[i]->GetEnable() == true && i < nbobject)
                    {
                            if(ListeObject[i]->OScript != NULL)ListeObject[i]->OScript->UpdateEverySecond();
                    }
                }
            }


            for(int i = 0; i < nbobject; i++)
                {
                    if(ListeObject[i]->GetEnable() == true && i < nbobject)
                    {
                        if(ListeObject[i]->OScript != NULL)ListeObject[i]->OScript->Update();
                    }
                }


            for(int i = 0; i < nbobject; i++)
                {
                    if(ListeObject[i]->GetEnable() == true)
                    {
                        if(ListeObject[i]->ORigibody != NULL)ListeObject[i]->ORigibody->AppliedForce();
                    }
                }

            UpdateRelativePosition();

            if(gamemap != NULL)gamemap->AfficherMap();

            for(int i = 0; i< nbobject; i++)
                {

                    if(ListeObject[i]->GetEnable() == true)
                    {
                        if(ListeObject[i]->ORender != NULL)ListeObject[i]->ORender->DrawObject();
                    }
                }

            if(EScript != NULL)EScript->Update();

            if(this->fps == true)PrintV(110,2,getFps());

            setFps(this->fpswish);
            Sleep(20);
            ML_display_vram();
            }

        while(this->run == false && this->execute == true && this->restart == false)
        {
             ML_clear_vram();
             ML_display_vram();
        }
    }

}while(this->restart == true);

    for(int i = 0; i < nbobject; i++)
        {
            delete ListeObject[i];
        }

return 1;

}

