#include "../C-Engine/CEngine/Object.hpp"
#include "../C-Engine/CEngine/Engine.hpp"
#include "../C-Engine/CEngine/Script.hpp"

#include "../C-Engine/CPhysics/Transforms.hpp"
#include "../C-Engine/CPhysics/Rigibodys.hpp"

#include "../C-Engine/CRendering/Render.hpp"
#include "../C-Engine/CRendering/Animation.hpp"
#include "../C-Engine/CRendering/Sprite.hpp"
#include "../C-Engine/CRendering/TilesMapping.hpp"

extern "C"
{
#include <fxlib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../C-Engine/CExtraLib/input.h"
#include "../C-Engine/CExtraLib/MonochromeLib.h"
#include "../C-Engine/CExtraLib/FonctionC.h"
}


Sprite::Sprite()
{
    this->Alpha = NULL;
    this->Beta = NULL;

    this->ReverseAlpha = NULL;
    this->ReverseBeta = NULL;
}

Sprite::Sprite(const unsigned char * v1, int sizex, int sizey , int bx , int by)
{
this->Beta=v1;
this->size_x=sizex;
this->size_y=sizey;

this->b_x = bx;
this->b_y = by;

this->Alpha = NULL;

this->ReverseAlpha = NULL;
this->ReverseBeta = NULL;
}

Sprite::Sprite(const unsigned char * v1, const unsigned char * v2, int sizex, int sizey , int bx, int by)
{
this->Beta=v1;
this->Alpha = v2;

this->size_x=sizex;
this->size_y=sizey;

this->b_x = bx;
this->b_y = by;

this->ReverseAlpha = NULL;
this->ReverseBeta = NULL;
}

void Sprite::CreateReverse( int brx , int bry)
{
    this->br_x = brx;
    this->br_y = bry;

    int taille, lx , decalage;

        unsigned char transit;
        unsigned char out ;

        decalage = size_x % 8;

    if(decalage == 0)lx = size_x/8;
    else lx = (size_x/8) + 1;

    taille = lx * size_y;


    if(Beta != NULL)
    {
        this->ReverseBeta = (unsigned char*) malloc(sizeof(unsigned char) * taille);

        for(int i = 0 ; i < size_y ; i++)
          for(int a = 0; a < lx; a++)
            ReverseBeta[i * lx + a] = Beta[ (i +1)*lx - (a+1)];

        for(int i = 0; i < taille ; i++)
         {

             transit = ReverseBeta[i];

             out = 0x0;


             if((transit & 0x80) != 0x0)  out = out | 0x01;
             if((transit & 0x40) != 0x0)  out = out | 0x02;
             if((transit & 0x20) != 0x0)  out = out | 0x04;
             if((transit & 0x10) != 0x0)  out = out | 0x08;
             if((transit & 0x08) != 0x0)   out = out | 0x10;
             if((transit & 0x04) != 0x0)   out = out | 0x20;
             if((transit & 0x02) != 0x0)   out = out | 0x40;
             if((transit & 0x01) != 0x0)   out = out | 0x80;

             ReverseBeta[i] = out;

         }

    if(decalage != 0)
    {
         for(int i = 0 ; i < size_y ; i++)
            for(int a = 0; a < lx; a++)
            {
                transit = ((unsigned char)ReverseBeta[i * lx + a] >> ( decalage));

                ReverseBeta[i * lx + a] = ((unsigned char)ReverseBeta[i * lx + a] << ( 8 - decalage));

                if(a != 0)ReverseBeta[i * lx + a - 1] = ReverseBeta[i * lx + a - 1] | transit;
            }

        }
    }


    if(Alpha != NULL)
    {
            this->ReverseAlpha = (unsigned char*) malloc(sizeof(unsigned char) * taille);

        for(int i = 0 ; i < size_y ; i++)
          for(int a = 0; a < lx; a++)
            ReverseAlpha[i * lx + a] = Alpha[ (i +1)*lx - (a+1)];

        for(int i = 0; i < taille ; i++)
         {

             transit = ReverseAlpha[i];

             out = 0x0;


             if((transit & 0x80) != 0x0)  out = out | 0x01;
             if((transit & 0x40) != 0x0)  out = out | 0x02;
             if((transit & 0x20) != 0x0)  out = out | 0x04;
             if((transit & 0x10) != 0x0)  out = out | 0x08;
             if((transit & 0x08) != 0x0)   out = out | 0x10;
             if((transit & 0x04) != 0x0)   out = out | 0x20;
             if((transit & 0x02) != 0x0)   out = out | 0x40;
             if((transit & 0x01) != 0x0)   out = out | 0x80;

             ReverseAlpha[i] = out;

         }

    if(decalage != 0)
    {
         for(int i = 0 ; i < size_y ; i++)
            for(int a = 0; a < lx; a++)
            {
                transit = ((unsigned char)ReverseAlpha[i * lx + a] >> ( decalage));

                ReverseAlpha[i * lx + a] = ((unsigned char)ReverseAlpha[i * lx + a] << ( 8 - decalage));

                if(a != 0)ReverseAlpha[i * lx + a - 1] = ReverseAlpha[i * lx + a - 1] | transit;
            }

        }


    }


}


void Sprite::DelSprite()
{
    this->Beta = NULL;
    this->Alpha = NULL;
    this->ReverseAlpha = NULL;
    this->ReverseBeta = NULL;
    this->size_x=0;
    this->size_y=0;
}

void Sprite::DrawSprite( int x , int y , int d)
{
    if(Beta != NULL)
    {
        if(Alpha != NULL)
        {
            ML_bmp_or_rotate((const unsigned char*) this->Alpha, x + b_x, y + b_y, this->size_x , this->size_y , d); // Affichage du canal alpha
            ML_bmp_and_rotate((const unsigned char*) this->Beta, x +b_x, y +b_y, this->size_x , this->size_y ,d); // Affichage du sprite en entier
        }
        else
        {
            ML_bmp_or_rotate((const unsigned char*) this->Beta, x +b_x , y +b_y, this->size_x , this->size_y , d); // Affichage du sprite en entier
        }

    }
}

void Sprite::DrawReverseSprite( int x , int y , int d)
{
    if(ReverseBeta != NULL)
    {
        if(ReverseAlpha != NULL)
        {
            ML_bmp_or_rotate((const unsigned char*) this->ReverseAlpha, x + br_x, y + br_y, this->size_x , this->size_y, d); // Affichage du canal alpha
            ML_bmp_and_rotate((const unsigned char*) this->ReverseBeta, x +br_x, y +br_y, this->size_x , this->size_y, d); // Affichage du sprite en entier
        }
        else
        {
            ML_bmp_or_rotate((const unsigned char*) this->ReverseBeta, x +br_x , y +br_y, this->size_x , this->size_y , d); // Affichage du sprite en entier
        }

    }
    else
    {
        DrawSprite( x , y , d);
    }
}

int Sprite::GetSizeY()
{
    return this->size_y;
}

int Sprite::GetSizeX()
{
    return this->size_x;
}

int Sprite::GetBlitX()
{
    return this->b_x;
}

int Sprite::GetBlitY()
{
    return this->b_y;
}
